"""
Supportive types for gadm library
"""

import enum

__all__ = ["AdministrativeDivisionLevel"]


class AdministrativeDivisionLevel(enum.IntEnum):
    """
    Level of Administrative Division specified by GADM Database
    """

    COUNTRY = 0
    STATE = 1
    DISTRICT = 2
    COMMUNE = 3
    MIN = COUNTRY
    MAX = COMMUNE



"""
Supportive utility methods for gadm library
"""

import os

import requests
import tqdm

__all__ = ["download_url"]


def download_url(url: str, file_path: str) -> None:
    """download file from url

    Parameters
    ----------
    url : str
        url of file to download
    file_path : str
        local path of the file
    """
    res0 = requests.head(url)
    assert "Content-Length" in res0.headers
    file_size = int(res0.headers["Content-Length"])

    if os.path.isfile(file_path) and os.path.getsize(file_path) == file_size:
        return

    res = requests.get(url, stream=True)

    with open(file_path, "wb") as _f:
        pbar = tqdm.tqdm(total=file_size, unit="B", unit_scale=True)
        for chunk in res.iter_content(chunk_size=1024):
            if chunk:
                _f.write(chunk)
            pbar.update(len(chunk))
        pbar.close()



import os
from io import open
from typing import Any, Dict

from setuptools import find_packages, setup

_PARENT_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
_PACKAGE_NAME = "gadm"

_PACKAGE_VARS: Dict[str, Any] = {}
exec(open(os.path.join(_PARENT_DIRECTORY, _PACKAGE_NAME, "version.py")).read(), _PACKAGE_VARS)

_LONG_DESCRIPTION = open(os.path.join(_PARENT_DIRECTORY, "README.md"), encoding="utf-8").read()
_INSTALL_REQUIRES = open(os.path.join(_PARENT_DIRECTORY, "requirements.txt")).read().splitlines()
_INSTALL_REQUIRES = [line for line in _INSTALL_REQUIRES if line and not line.startswith("#")]


def main():
    setup(
        name=_PACKAGE_NAME,
        version=_PACKAGE_VARS["__version__"],
        description="package to get boundaries data from GADM,  the database of global Administrative Areas.",
        long_description=_LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        author="Ba Tran",
        url="https://github.com/xmba15/gadm",
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
        ],
        packages=find_packages(exclude=["tests"]),
        install_requires=_INSTALL_REQUIRES,
    )


if __name__ == "__main__":
    main()



"""
Module to download GADM data, the database of global Administrative Areas.
"""

import logging
import os
from typing import List, Optional

import fiona
import geopandas as gpd
import pycountry as pct

from gadm.custom_types import AdministrativeDivisionLevel as ADLevel
from gadm.utils import download_url

__all__ = ["GADMDownloader"]


class GADMDownloader:
    """
    Data downloader of GADM, the database of global Administrative Areas.
    Official Website: https://gadm.org/
    """

    __SUPPORTED_VERSIONS = {"4.0": "40"}
    __ROOT_URL_TEMPLATE = "https://geodata.ucdavis.edu/gadm/gadm{version}/gpkg/"
    __DATA_PATH_TEMPLATE = "gadm{version_wo_dot}_{country_alpha_3}.gpkg"
    __CACHED_DIR = os.path.join(os.path.expanduser("~"), ".gadm")

    def __init__(self, version: str = "4.0", logger: logging.Logger = logging.getLogger("gadm_downloader.db")):
        """
        Parameters
        ----------
        version : str
            version of gadm data, currently only support version 4.0
        logger : logging.Logger
            logger instance
        """
        assert version in GADMDownloader.__SUPPORTED_VERSIONS, f"{version} is not a supported version"
        self._version = version
        self._url_root_dir = GADMDownloader.__ROOT_URL_TEMPLATE.format(
            version=self._version,
        )
        self._logger = logger

        if not os.path.isdir(GADMDownloader.__CACHED_DIR):
            os.makedirs(GADMDownloader.__CACHED_DIR)

    def get_shape_data_by_country(
        self, country: pct.ExistingCountries.data_class_base, ad_level: int
    ) -> Optional[gpd.GeoDataFrame]:
        """get shape data as a GeoDataFrame by country object

        Parameters
        ----------
        country : Country
            object of Country defined in pycountry library
        ad_level : int
            administrative division level

        Returns
        -------
        Optional[gpd.GeoDataFrame]
            optional GeoDataFrame shape
        """

        data_path = GADMDownloader.__DATA_PATH_TEMPLATE.format(
            version_wo_dot=GADMDownloader.__SUPPORTED_VERSIONS[self._version], country_alpha_3=country.alpha_3
        )
        abs_data_path = os.path.join(GADMDownloader.__CACHED_DIR, data_path)
        try:
            download_url(
                self._url_root_dir + data_path,
                abs_data_path,
            )
        except Exception as e:
            self._logger.error(f"failed to fetch data for {country.common_name} with error {e}")
            return None

        layers: List[str] = sorted(fiona.listlayers(abs_data_path))
        if len(layers) < ad_level:
            self._logger.info("gadm does not have data for level {ad_level} of {country.common_name}")
            return None

        return gpd.read_file(abs_data_path, layer=layers[ad_level])

    def get_shape_data_by_country_name(self, country_name: str, ad_level: int) -> Optional[gpd.GeoDataFrame]:
        """get shape data as a GeoDataFrame by country name

        Parameters
        ----------
        country_name : str
            country name
        ad_level : int
            administrative division level

        Returns
        -------
        Optional[gpd.GeoDataFrame]
            optional GeoDataFrame shape
        """

        assert ADLevel.MIN <= ad_level <= ADLevel.MAX
        try:
            country = pct.countries.lookup(country_name)
        except LookupError:
            self._logger.error(f"could not find data for {country_name}")
            return None

        return self.get_shape_data_by_country(country, ad_level)
